/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: VO BaseAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: VOAction.h,v 1.7 2005/10/14 10:32:54 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_2_2_4 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_VOACTION_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_VOACTION_H_

#include "glite/data/transfer/agent/action/Action.h"
#include "glite/data/transfer/agent/action/ExecuteException.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/TransferDAO.h"
#include "glite/data/transfer/agent/dao/vo/ChannelDAO.h"
#include "glite/data/transfer/agent/dao/DAOContext.h"

#include <string>
#include <boost/scoped_ptr.hpp>
#include <set>

namespace log4cpp{
    class Category;
}

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {
        
/**
 * Base Class For all the VO Actions
 */
class VOAction : public Action {
public:    
    
    /**    
     * set DAOContext
     */
    virtual void context(const std::string& vo_name, dao::DAOContext& ctx);
    
    /**    
     * Get DAOContext
     */
    virtual dao::DAOContext& context() /*throw (ExecuteException)*/;
    
    /**
     * Destructor
     */
    virtual ~VOAction();

    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") /*throw (ExecuteException)*/;

protected:

    /**
     * Constructor
     */
    VOAction(const std::string& name);

    /**
     * Call the virtual method to execute the action (Template Pattern)
     */
    virtual void exec(const std::string& params, unsigned int& failures) /*throw (AgentException)*/ = 0;

    /**
     * Update Jobs Status
     */
    void updateJobs(const std::set<std::string>& ids, bool enableFinalStates = false) /*throw (DAOException)*/;

    /**
     * Send a Fail Event for a file
     */
    void failFile(const std::string& file, model::Error::Category c, const std::string& reason) /*throw ()*/;

    /**
     * Send a Fail Event for all the files
     */
    void failAllFiles(const std::vector<std::string>& files, model::Error::Category c, const std::string& reason) /*throw ()*/;
    
    /**
     * Get The Proxy Certficate for a Job
     */
    void getProxyForJob(model::Job& job, std::string& proxy_file) /* throw(AgentException)*/;

    /**
     * Normalize a Surl accordingly with the VO policy
     */
    std::string normalizeSurl(const std::string& surl, const std::string& srm_endpoint) /* throw()*/;

private:
    
    /**
     * Default Constructor. Not Implemented
     */
    VOAction();

    /**
     * Copy Constructor. Not Implemented
     */
    VOAction(const VOAction&);

protected:
    
    typedef dao::vo::JobDAO       JobDAO;
    typedef dao::vo::FileDAO      FileDAO;
    typedef dao::vo::TransferDAO  TransferDAO;
    typedef dao::vo::ChannelDAO   ChannelDAO;

    /**    
     * Get a JobDAO Instance
     */
    JobDAO& jobDAO() /*throw (ExecuteException)*/;
    
    /**    
     * Get a FileDAO Instance
     */
    FileDAO& fileDAO() /*throw (ExecuteException)*/;

    /**    
     * Get a TransferDAO Instance
     */
    TransferDAO& transferDAO() /*throw (ExecuteException)*/;

    /**    
     * Get a ChannelDAO Instance
     */
    ChannelDAO& channelDAO() /*throw (ExecuteException)*/;

    /**
     * Get The VO Name
     */
    const std::string& voName() { return m_voName;}
    
    /**
     * The Logger
     */
    class log4cpp::Category& m_logger;

private:
    
    /**
     * The DAO Context
     */
    dao::DAOContext* m_context;

    /**
     * The Vo Name
     */
    std::string m_voName;

    /**
     * The Job DAO
     */
    boost::scoped_ptr<JobDAO> m_jobDAO;

    /**
     * The File DAO
     */
    boost::scoped_ptr<FileDAO> m_fileDAO;
    
    /**
     * The Transfer DAO
     */
    boost::scoped_ptr<TransferDAO> m_transferDAO;
    
    /**
     * The Channel DAO
     */
    boost::scoped_ptr<ChannelDAO> m_channelDAO;
    
};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_BASEACTION_H_
