/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: CatalogService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogService.h,v 1.3 2005/07/27 10:59:17 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_2_2_4 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICE_H_

#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"
#include <vector>
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * CatalogService Interface.
 * Define the interface for registering and retrieving logical and pohysical 
 * names from a catalog Service
 */
class CatalogService{
public:
    typedef std::pair<std::string,std::string> StringPair;
    typedef std::vector<std::string>           StringArray;
    typedef std::vector<StringPair>            StringPairArray;
    /**
     * Destructor
     */
    virtual ~CatalogService(){}
        
    /**
     * Check that the user permission is allowed to read and replicate the 
     * requested files
     * @param logicals [IN] The list of input Logical Name to be checked. It can
     * be either LFN or GUID, but the same type should be used within a request 
     * @return true if authorization is granted for all files
     * @throws CatalogServiceException in case of errors. The exception may 
     * contains the list of the logical name of the fails that fails. If that 
     * list is empty, all the files should be considered failed
     */
    virtual bool checkPermissions(const StringArray& logicals) /*throw (CatalogServiceException)*/ = 0;
    
    /**
     * List the names of all the replicas associated to the given Logical Name
     * @param logicals [IN] The list of input Logical Name.  It can be 
     * either LFN or GUID, but the same type should be used within a request 
     * @param surls [OUT] the array that will store the name of all the replicas
     * @throws CatalogServiceException in case of errors. The exception may 
     * contains the list of the logical name of the fails that fails. If that 
     * list is empty, all the files should be considered failed
     */
    virtual void listSurls(const StringArray& logicals, 
                                 std::vector<StringArray>& surls) /*throw (CatalogServiceException)*/ = 0;
    
    /**
     * Register the name of new replicas 
     * @param names [IN] The list of The Logical Name and SURL pairs. The 
     * Logical Name can be either LFN or GUID, but the same type should be used 
     * within a request
     * @throws CatalogServiceException in case of errors. The exception may 
     * contains the list of the logical name of the fails that fails. If that 
     * list is empty, all the files should be considered failed
     */
    virtual void registerSurls(const StringPairArray& names) /*throw (CatalogServiceException)*/ = 0;
                               
protected:
    /**
     * Constructor
     */
    CatalogService(){}

private:
    /** @link dependency
     * @label throws*/
    /*# CatalogServiceException lnkCatalogServiceException; */
};

} // End namespace catalog
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICE_H_
