/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: CatalogServiceFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogServiceFactory.h,v 1.3 2005/07/22 09:22:34 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_2_2_4 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEFACTORY_H_

#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"
#include "glite/data/transfer/agent/catalog/CatalogService.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * CatalogServiceFactory interface
 * @interface
 */
class CatalogServiceFactory  {
public:

    /**
     * Create a CatalogService instance
     * @param endpoint [IN] the endpoint of the service to contact 
     * @param type [IN] the type of the service to create (can be the value 
     * returned either by globalCatalogServiceType or localCatalogServiceType
     * methods)
     * @param cred [IN] the name of the file containing the credential to be
     * used to contact the service. If this value is empty, the default will 
     * apply
     * @return a new instance of a CatalogService
     * @throws CatalogServiceException in case of error
     */
    virtual CatalogService * create(const std::string& endpoint, const std::string& type, const std::string& cred = "") /*throw (CatalogServiceException)*/ = 0; 

    /**
     * Return the Global Catalog Service Type (if any) to be used to query 
     * ServiceDiscovery. If the Catalog Deployment model doesn't support the
     * global scenario, an emty string should be returned
     * @return the Global Catalog Service Type to be used to query 
     */
    virtual std::string globalCatalogServiceType() /*throw ()*/ = 0; 

    /**
     * Return the Local Catalog Service Type (if any) to be used to query 
     * ServiceDiscovery. If the Catalog Deployment model doesn't support the
     * local scenario, an emty string should be returned
     * @return the local Catalog Service Type to be used to query 
     */
    virtual std::string localCatalogServiceType() /*throw ()*/ = 0; 

    /**
     * Return the configured CatalogServiceFactory Instance. Please note that 
     * this method doesn't allocate the instance (it's not a singleton)
     * @return the configued CatalogServiceFactory 
     * @throws CatalogServiceException in case of no CatalogServiceFactory is
     * configured
     */
    static CatalogServiceFactory * getInstance() /*throw (CatalogServiceException)*/; 
   
protected:

    /**
     * Constructor
     */ 
    CatalogServiceFactory();

    /**
     * Destructor
     */ 
    virtual ~CatalogServiceFactory();

private:

    /**
     * The registered Instance
     */
    static CatalogServiceFactory * s_instance;
    
    /** @link dependency 
     * @label creates*/
    /*# CatalogService lnkCatalogService; */

    /** @link dependency 
     * @label throws*/
    /*# CatalogServiceException lnkCatalogServiceException; */
};

} // End namespace catalog
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEFACTORY_H_
