/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: TransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferService.h,v 1.5 2005/10/07 15:07:22 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_2_2_4 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_

#include "glite/data/transfer/agent/glite-types.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/agent/ts/TransferException.h"

#include <vector>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

/**
 * TransferService interface. This class defines the interface to integrate 
 * different TransferService implementations (Adapter Pattern)
 * @interface
 */
class TransferService {
public:

    /**
     * TransferRequest Struct
     */
    struct TransferRequest{
        typedef std::pair<std::string,std::string> SurlPair;
        typedef std::vector<SurlPair> SurlPairArray;
        SurlPairArray    surls;
        std::string      parameters;
        unsigned long    streams;
        std::string      src_srm;
        std::string      dest_srm;
    };

    /**
     * TransferResponse Struct
     */
    struct TransferResponse{
        std::string               request_id;
        std::vector<unsigned int> transfer_ids;
    };
    
    /**
     * TransferObject Array Type
     */
    typedef std::vector<glite::data::transfer::agent::model::Transfer*> TransferInfoArray;

    /**
     * Destructor
     */
    virtual ~TransferService(){}

    /**
     * Return the name of the Channel the TransferService is responsible for
     * @return the name of the Channel the TransferService is responsible for
     * @throws TransferException in case the channel name is not set
     */
    virtual std::string channel() /*throw (TransferException)*/ = 0;

    /**
     * Get the number of max concurrent transfer requests the service can support
     * @return the number of max concurrent transfer requests the service can support
     */
    virtual unsigned long max() = 0;

    /**
     * Get the number of max transfers that can be processed in the same request
     * @return number of max transfers that can be processed in the same request
     */
    virtual unsigned long maxRequestSize() = 0;

    /**
     * Submit Transfers
     * @param req [INOUT] the request containing the list of SURLs pair to
     * transfer.
     * @param out [OUT] The result  object containing the request identifier and
     * @param credentials [IN] the credentials to be used to perform the action
     * a list of transfer identifier
     * @throws TransferException in case of error
     */
    virtual void submit(const TransferRequest& req, TransferResponse& out, const std::string& credentials) /*throw (TransferException)*/ = 0;

    /**
     * Cancel a Pending Transfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case of error
     */
    virtual void revoke(const std::string& requestId, const std::string& credentials) /*throw (TransferException)*/ = 0;
    
    /**
     * Get the status of a FileTransfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param objs [INOUT] the array that will containing the information for
     * each transfer involved on the request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case of error
     */
    virtual void state(const std::string& requestId, TransferInfoArray& objs, const std::string& credentials) /*throw (TransferException)*/ = 0;
    
    /**
     * Clean-up the information concerning a completed Transfer 
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case the requested Transfer request 
     * doesn't exist or is not completed
     */
    virtual void clean(const std::string& requestId, const std::string& credentials) /*throw (TransferException)*/ = 0;

    /**
     * Trace a Transfer
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param out [IN] the buffer where to store the transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case the requested Transfer request 
     * doesn't exist or is not completed
     */
    virtual void trace(const std::string& requestId, std::ostream& out, const std::string& credentials) /*throw (TransferException)*/ = 0;

    /**
     * Get the number of all the pending transfer requests on the channel
     * @return the number of all the pending transfer requests on the channel
     * @throws TransferException in case of error
     */
    virtual unsigned long count() /*throw (TransferException)*/ = 0;

private:

    /** @link dependency */
    /*# File lnkFile; */

    /** @link dependency 
     * @label throws*/
    /*# TransferException lnkTransferException; */
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_
